@@ -1,23 +1,28 @@
-# $Id: Svg.pm 2114 2006-08-29 23:51:35Z rvosa $
-# Subversion: $Rev: 192 $
+# $Id: Svg.pm 1128 2009-06-16 07:26:07Z rvos $
 package Bio::Phylo::Treedrawer::Svg;
 use strict;
-use constant PI => '3.14159265358979323846';
-use SVG (
+use Bio::Phylo::Util::CONSTANT 'looks_like_hash';
+use Bio::Phylo::Util::Exceptions 'throw';
+
+eval { require SVG };
+if ( $@ ) {
+	throw 'ExtensionError' => "Error loading the SVG extension: $@";
+}
+SVG->import(
     '-nocredits' => 1,
     '-inline'    => 1,
     '-indent'    => '    ',
 );
-my @fields = qw(TREE SVG DRAWER);
 
-# One line so MakeMaker sees it.
-use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
+my @fields = qw(TREE SVG DRAWER);
+my $PI = '3.14159265358979323846';
+
 my %colors;
 
 =head1 NAME
 
-Bio::Phylo::Treedrawer::Svg - Creates svg tree drawings. No serviceable parts
-inside.
+Bio::Phylo::Treedrawer::Svg - Graphics format writer used by treedrawer, no serviceable parts
+inside
 
 =head1 DESCRIPTION
 
@@ -46,11 +51,7 @@ the node, and branch leading up to it, in red.)
 sub _new {
     my $class = shift;
     my $self = {};
-    my %opt;
-    eval { %opt = @_; };
-    if ($@) {
-        Bio::Phylo::Util::Exceptions::OddHash->throw( error => $@ );
-    }
+    my %opt = looks_like_hash @_;
     $self->{'TREE'}   = $opt{'-tree'};
     $self->{'DRAWER'} = $opt{'-drawer'};
     return bless $self, $class;
@@ -71,11 +72,10 @@ sub _new {
 
 sub _draw {
     my $self = shift;
-    $self->{'SVG'} = SVG->new(
-        'width'  => $self->{'DRAWER'}->get_width,
-        'height' => $self->{'DRAWER'}->get_height
-    );
-    $self->{'SVG'}->tag( 'style', type => 'text/css' )
+    my $drawer = $self->{'DRAWER'};    
+    $self->{'SVG'} = SVG->new( 'width' => $drawer->get_width, 'height' => $drawer->get_height );
+    my $svg = $self->{'SVG'};
+    $svg->tag( 'style', type => 'text/css' )
       ->CDATA( "\n\tpolyline { fill: none; stroke: black; stroke-width: 2 }\n"
           . "\tpath { fill: none; stroke: black; stroke-width: 2 }\n"
           . "\tline { fill: none; stroke: black; stroke-width: 2 }\n"
@@ -87,48 +87,28 @@ sub _draw {
           . "\ttext.scale_label    {}\n"
           . "\tline.scale_major    {}\n"
           . "\tline.scale_minor    {}\n" );
-    foreach my $node ( @{ $self->{'TREE'}->get_entities } ) {
-        my $cx = int $node->get_generic('x');
-        my $cy = int $node->get_generic('y');
-        my $r  = int $self->{'DRAWER'}->get_node_radius;
-        my $x  =
-          int( $node->get_generic('x') +
-              $self->{'DRAWER'}->get_text_horiz_offset );
-        my $y =
-          int(
-            $node->get_generic('y') + $self->{'DRAWER'}->get_text_vert_offset );
+    for my $node ( @{ $self->{'TREE'}->get_entities } ) {
+        my $name = $node->get_name || ' ';
+        my $url  = $node->get_url;         
+        $name =~ s/_/ /g;
+        $name =~ s/^'(.*)'$/$1/;
+        $name =~ s/^"(.*)"$/$1/;        
+        my ( %class, $r );
+        my $is_terminal  = $node->is_terminal;
+        $class{'circle'} = $is_terminal ? 'taxon_circle' : 'node_circle';
+        $class{'text'}   = $is_terminal ? 'taxon_text'   : 'node_text';
+        $r = $is_terminal ? int($drawer->get_tip_radius) : int($drawer->get_node_radius);
+        my %circle = ( 'cx' => int($node->get_x), 'cy' => int($node->get_y), 'r' => $r, 'class' => $class{'circle'} );
+        my $x = int( $node->get_x + $drawer->get_text_horiz_offset );
+        my $y = int( $node->get_y + $drawer->get_text_vert_offset );
+        my %text = ( 'x' => $x, 'y' => $y, 'class' => $class{'text'} );
+        my %args;
         if ( my $style = $node->get_generic('svg') ) {
-            $self->{'SVG'}->tag(
-                'circle',
-                'cx'    => $cx,
-                'cy'    => $cy,
-                'r'     => $r,
-                'style' => $style,
-                'class' => $node->is_terminal ? 'taxon_circle' : 'node_circle',
-            );
-            $self->{'SVG'}->tag(
-                'text',
-                'x'     => $x,
-                'y'     => $y,
-                'style' => $style,
-                'class' => $node->is_terminal ? 'taxon_text' : 'node_text',
-            )->cdata( $node->get_name ? $node->get_name : ' ' );
-        }
-        else {
-            $self->{'SVG'}->tag(
-                'circle',
-                'cx'    => $cx,
-                'cy'    => $cy,
-                'r'     => $r,
-                'class' => $node->is_terminal ? 'taxon_circle' : 'node_circle',
-            );
-            $self->{'SVG'}->tag(
-                'text',
-                'x'     => $x,
-                'y'     => $y,
-                'class' => $node->is_terminal ? 'taxon_text' : 'node_text',
-            )->cdata( $node->get_name ? $node->get_name : ' ' );
+        	%args = ( 'style' => $style );
         }
+		my $invocant = $url ? $svg->tag( 'a', 'xlink:href' => $url ) : $svg;
+        $invocant->tag( 'circle', %circle, %args );
+        $svg->tag( 'text', %text, %args )->cdata( $name );        
         if ( $node->get_parent ) {
             $self->_draw_line($node);
         }
@@ -157,15 +137,15 @@ sub _draw {
 sub _draw_pies {
     my $self = shift;
     foreach my $node ( @{ $self->{'TREE'}->get_entities } ) {
-        my $cx = int $node->get_generic('x');
-        my $cy = int $node->get_generic('y');
+        my $cx = int $node->get_x;
+        my $cy = int $node->get_y;
         my $r  = int $self->{'DRAWER'}->get_node_radius;
         my $x  =
-          int( $node->get_generic('x') +
+          int( $node->get_x +
               $self->{'DRAWER'}->get_text_horiz_offset );
         my $y =
           int(
-            $node->get_generic('y') + $self->{'DRAWER'}->get_text_vert_offset );
+            $node->get_y + $self->{'DRAWER'}->get_text_vert_offset );
         if ( my $pievalues = $node->get_generic('pie') ) {
             my @keys  = keys %{$pievalues};
             my $start = -90;
@@ -183,12 +163,12 @@ sub _draw_pies {
                 my $slice = $pievalues->{ $keys[$i] } / $total * 360;
                 my $color = $colors{ $keys[$i] };
                 if ( not $color ) {
-                    my $gray = int( ( ( $i + 1 ) / scalar @keys ) * 256 );
+                    my $gray = int( ( $i / $#keys ) * 256 );
                     $color = sprintf 'rgb(%d,%d,%d)', $gray, $gray, $gray;
                     $colors{ $keys[$i] } = $color;
                 }
                 my $do_arc  = 0;
-                my $radians = $slice * PI / 180;
+                my $radians = $slice * $PI / 180;
                 $do_arc++ if $slice > 180;
                 my $radius = $r - 2;
                 my $ry     = ( $radius * sin($radians) );
@@ -223,14 +203,17 @@ sub _draw_scale {
     my $drawer  = $self->{'DRAWER'};
     my $svg     = $self->{'SVG'};
     my $tree    = $self->{'TREE'};
+    my $root    = $tree->get_root;
+    my $rootx   = $root->get_x;
+    my $height  = $drawer->get_height;
     my $options = $drawer->get_scale_options;
-    if ($options) {
+    if ( $options ) {
         my ( $major, $minor ) = ( $options->{'-major'}, $options->{'-minor'} );
         my $width = $options->{'-width'};
         if ( $width =~ m/^(\d+)%$/ ) {
             $width =
-              ( $1 / 100 ) * ( $tree->get_tallest_tip->get_generic('x') -
-                  $tree->get_root->get_generic('x') );
+              ( $1 / 100 ) * ( $tree->get_tallest_tip->get_x -
+                  $rootx );
         }
         if ( $major =~ m/^(\d+)%$/ ) {
             $major = ( $1 / 100 ) * $width;
@@ -239,58 +222,44 @@ sub _draw_scale {
             $minor = ( $1 / 100 ) * $width;
         }
         my $major_text  = 0;
-        my $major_scale =
-          ( $major / $width ) * $tree->get_root->calc_max_path_to_tips;
+        my $major_scale = ( $major / $width ) * $root->calc_max_path_to_tips;
         $svg->line(
             'class' => 'scale_bar',
-            'x1'    => $tree->get_root->get_generic('x'),
-            'y1'    => ( $drawer->get_height - 5 ),
-            'x2'    => $tree->get_root->get_generic('x') + $width,
-            'y2'    => ( $drawer->get_height - 5 ),
+            'x1'    => $rootx,
+            'y1'    => ( $height - 5 ),
+            'x2'    => $rootx + $width,
+            'y2'    => ( $height - 5 ),
         );
         $svg->tag(
             'text',
-            'x' => (
-                $tree->get_root->get_generic('x') + $width +
-                  $drawer->get_text_horiz_offset
-            ),
-            'y'     => ( $drawer->get_height - 5 ),
+            'x'     => ( $rootx + $width + $drawer->get_text_horiz_offset ),
+            'y'     => ( $height - 5 ),
             'class' => 'scale_label',
         )->cdata( $options->{'-label'} ? $options->{'-label'} : ' ' );
-        for (
-            my $i = $tree->get_root->get_generic('x') ;
-            $i <= ( $tree->get_root->get_generic('x') + $width ) ;
-            $i += $major
-          )
-        {
+        for ( my $i = $rootx; $i <= ( $rootx + $width ); $i += $major ) {
             $svg->line(
                 'class' => 'scale_major',
                 'x1'    => $i,
-                'y1'    => ( $drawer->get_height - 5 ),
+                'y1'    => ( $height - 5 ),
                 'x2'    => $i,
-                'y2'    => ( $drawer->get_height - 25 ),
+                'y2'    => ( $height - 25 ),
             );
             $svg->tag(
                 'text',
                 'x'     => $i,
-                'y'     => ( $drawer->get_height - 35 ),
+                'y'     => ( $height - 35 ),
                 'class' => 'major_label',
             )->cdata( $major_text ? $major_text : ' ' );
             $major_text += $major_scale;
         }
-        for (
-            my $i = $tree->get_root->get_generic('x') ;
-            $i <= ( $tree->get_root->get_generic('x') + $width ) ;
-            $i += $minor
-          )
-        {
+        for ( my $i = $rootx; $i <= ( $rootx + $width ); $i += $minor ) {
             next if not $i % $major;
             $svg->line(
                 'class' => 'scale_minor',
                 'x1'    => $i,
-                'y1'    => ( $drawer->get_height - 5 ),
+                'y1'    => ( $height - 5 ),
                 'x2'    => $i,
-                'y2'    => ( $drawer->get_height - 15 ),
+                'y2'    => ( $height - 15 ),
             );
         }
     }
@@ -317,9 +286,9 @@ sub _draw_legend {
         my $draw      = $self->{'DRAWER'};
         my @keys      = keys %colors;
         my $increment =
-          ( $tree->get_tallest_tip->get_generic('x') -
-              $tree->get_root->get_generic('x') ) / scalar @keys;
-        my $x = $tree->get_root->get_generic('x') + 5;
+          ( $tree->get_tallest_tip->get_x -
+              $tree->get_root->get_x ) / scalar @keys;
+        my $x = $tree->get_root->get_x + 5;
         foreach my $key (@keys) {
             $svg->rectangle(
                 'x'      => $x,
@@ -344,7 +313,7 @@ sub _draw_legend {
         $svg->tag(
             'text',
             'x' => (
-                $tree->get_tallest_tip->get_generic('x') +
+                $tree->get_tallest_tip->get_x +
                   $draw->get_text_horiz_offset
             ),
             'y'     => ( $draw->get_height - 80 ),
@@ -368,11 +337,12 @@ sub _draw_legend {
 
 sub _draw_line {
     my ( $self, $node ) = @_;
+    my $pnode      = $node->get_parent;
     my $node_hash  = $node->get_generic;
-    my $pnode_hash = $node->get_parent->get_generic;
+    my $pnode_hash = $pnode->get_generic;
     my ( $x1, $x2, $style ) =
-      ( int $pnode_hash->{'x'}, int $node_hash->{'x'}, $node_hash->{'svg'} );
-    my ( $y1, $y2 ) = ( int $pnode_hash->{'y'}, int $node_hash->{'y'} );
+      ( int $pnode->get_x, int $node->get_x, $node_hash->{'svg'} );
+    my ( $y1, $y2 ) = ( int $pnode->get_y, int $node->get_y );
     if ( $self->{'DRAWER'}->get_shape eq 'CURVY' ) {
         my $points = qq{M$x1,$y1 C$x1,$y2 $x2,$y2 $x2,$y2};
         if ($style) {
@@ -429,52 +399,13 @@ to learn how to create tree drawings.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>.
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Svg.pm 2114 2006-08-29 23:51:35Z rvosa $
-
-=head1 AUTHOR
-
-Rutger A. Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger A. Vos, All Rights Reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as Perl
-itself.
+ $Id: Svg.pm 1128 2009-06-16 07:26:07Z rvos $
 
 =cut
 
